@@ -2,7 +2,7 @@
  * Ghost Port Scan is distributed under the GPL (see COPYING)
  *
  * Homepage: gps.sourceforge.net
- * Contact.: whitehat@altern.org
+ * Contact.: teardrop@altern.org
  *
  */
 
@@ -121,7 +121,7 @@ int main(int argc, char **argv)
 	// load known services in memory
 	st=services_load();
 
-	fprintf(stdout, "Ghost Port Scan version %s by whitehat@altern.org (gps.sourceforge.net)\n", GPS_VERSION);
+	fprintf(stdout, "Ghost Port Scan version %s by teardrop@altern.org (gps.sourceforge.net)\n", GPS_VERSION);
 
 	// take a look to the arguments:
 	if(argc<3) usage(argv[0]);
@@ -331,7 +331,7 @@ int main(int argc, char **argv)
 			}
 			else libnet_error(LIBNET_ERR_FATAL, "All attemps to get a suitable network device failed. You should try the -i option to specify the device to use.\n");
 #ifdef HAVE_BSD
-			if(strncmp("ed", dev.name, 2) && strncmp("lo", dev.name, 2) && strncmp("ne", dev.name, 2))
+			if(strncmp("fxp", dev.name, 3) && strncmp("my", dev.name, 2) && strncmp("ed", dev.name, 2) && strncmp("lo", dev.name, 2) && strncmp("ne", dev.name, 2))
 #else
 			if(strncmp("et", dev.name, 2) && strncmp("lo", dev.name, 2))
 #endif
