@@ -1,13 +1,10 @@
 #ifndef MAC_ARP_H
 #define MAC_ARP_H
 
-#include<sys/types.h>
-
-#define FROUTE	"/proc/net/route"
+#include "gps.h"
 
 #define ARP_TIMEOUT 1000
 
-
 typedef struct
 {
 	struct sockaddr_in arp_pa;
@@ -17,10 +14,7 @@ typedef struct
 	char arp_dev[16];
 } my_arpreq;
 
-
-int ahtoi(char *ptr);
-int power(int i, int j);
-u_int32_t get_gateway_ip(char *fn_route);
+// functions
 int send_ARP_request(char *dev, u_int32_t *srcip, u_int32_t *dstip, u_char *src_mac, u_char *dst_mac);
 int get_mac(u_int32_t ip, char *mac, char *dev);
 
