@@ -1,67 +1,66 @@
+#include "route.h"
+
 #include<sys/ioctl.h>
 #include<sys/socket.h>
 #include<libnet.h>
 #include<stdio.h>
+#include<errno.h>
 #include<stdlib.h>
 #include<string.h>
 
-#include "route.h"
-
 /*
- * Note that SIOCGIFxxxx calls
- * produce weird results. I had
- * to create a suitable structure
- * (my_ifreq) to have them working.
+ * some parts are taken from nmap
  */
 devlist *get_localdevs()
 {
-	struct ifconf ifc;
-	my_ifreq ifr_nm, *ifr;
-	int sd, i;
 	char buffer[10240]; /* weird size ? -- taken from nmap */
+	struct ifconf ifc;
 	static devlist localdevs;
+	struct ifreq ifr_nm, *ifr;
+	int sd, i, len;
 
 	localdevs.nb=0;
 	localdevs.dev_list=NULL;
-/*
- *	realloc() seems to not need a previously allocated block
- *	uncomment this if you have trouble with realloc()
- *
-	if(!(localdevs.dev_list=(netdev *)malloc(sizeof(netdev))))
-		{ fprintf(stderr, "malloc: memory allocation failed\n"); return(NULL); }
-*/
 
 	if((sd=socket(AF_INET, SOCK_DGRAM, 0))<0) 
-		{ fprintf(stderr, "socket: can't open a socket\n"); return(NULL); }
+		{ perror("get_localdevs: socket"); return(NULL); }
 
 	ifc.ifc_len=sizeof(buffer);
 	ifc.ifc_ifcu.ifcu_buf=buffer;
 	// get devices and their addresses
 	if(ioctl(sd, SIOCGIFCONF, &ifc)<0)
-		{ fprintf(stderr, "ioctl: can't get the network interface list\n"); close(sd); return(NULL); }
+		{ perror("get_localdevs: ioctl"); close(sd); return(NULL); }
 
 	if(!ifc.ifc_len)
 		{ fprintf(stderr, "According to SIOCGIFCONF, you don't have any network interface\n"); close(sd); return(NULL); }
 
-	for(ifr=(my_ifreq *)buffer; ifr && *(char *)ifr && (char *)ifr<buffer+ifc.ifc_len; ifr++)
+#ifndef HAVE_SOCKADDR_SA_LEN
+	len=sizeof(struct ifreq);
+#endif
+
+	for(ifr=(struct ifreq *)buffer; ifr && *(char *)ifr && (char *)ifr<buffer+ifc.ifc_len; (*(char **)&ifr)+=len)
 	{
-		if(!ifr->ifr_ifrn.ifrn_name[0]) continue;
+#ifdef HAVE_SOCKADDR_SA_LEN
+		len=ifr->ifr_addr.sa_len+sizeof(ifr->ifr_name);
+#endif
+		if(!ifr->ifr_name[0] || ((struct sockaddr_in *)&ifr->ifr_addr)->sin_family!=AF_INET) continue;
 		if(!(localdevs.dev_list=(netdev *)realloc(localdevs.dev_list, (localdevs.nb+1)*sizeof(netdev))))
-			{ fprintf(stderr, "realloc: memory reallocation failed\n"); close(sd); return(NULL); }
-		strncpy(localdevs.dev_list[localdevs.nb].name, ifr->ifr_ifrn.ifrn_name, IFNAMSIZ);
+			{ perror("get_localdevs: realloc"); close(sd); return(NULL); }
+		memcpy(localdevs.dev_list[localdevs.nb].name, ifr->ifr_name, IFNAMSIZ);
 		localdevs.dev_list[localdevs.nb].name[IFNAMSIZ-1]='\0';
-		localdevs.dev_list[localdevs.nb].ip=ifr->ifr_ifru.ifru_addr.sin_addr.s_addr;
-		localdevs.dev_list[localdevs.nb++].netmask=ifr->ifr_ifru.ifru_netmask.sin_addr.s_addr;
+		localdevs.dev_list[localdevs.nb].ip=((struct sockaddr_in *)(&ifr->ifr_addr))->sin_addr.s_addr;
+		localdevs.nb++;
 	}
+
 	// get netmasks
 	for(i=0; i<localdevs.nb; i++)
 	{
 		bzero(&ifr_nm, sizeof(struct ifreq));
-		ifr_nm.ifr_netmask.sin_family=AF_INET;
-		strncpy(ifr_nm.ifr_name, localdevs.dev_list[i].name, IFNAMSIZ);
+		((struct sockaddr_in *)(&ifr_nm.ifr_addr/*netmask*/))->sin_family=AF_INET;
+		memcpy(ifr_nm.ifr_name, localdevs.dev_list[i].name, IFNAMSIZ);
 		if(ioctl(sd, SIOCGIFNETMASK, (char *)&ifr_nm)<0)
-			{ fprintf(stderr, "ioctl: can't get a netmask\n"); close(sd); return(NULL); }
-		localdevs.dev_list[i].netmask=ifr_nm.ifr_netmask.sin_addr.s_addr;
+			{ perror("get_localdevs: ioctl"); close(sd); return(NULL); }
+		localdevs.dev_list[i].netmask=((struct sockaddr_in *)(&ifr_nm.ifr_addr/*netmask*/))->sin_addr.s_addr;
 	}
 	close(sd);
 	return(&localdevs);
@@ -89,7 +88,7 @@ netdev *get_suitable_dev(devlist *dl, u_int32_t target_ip)
 	for(i=0; i<dl->nb; i++)
 		if((dl->dev_list[i].ip & dl->dev_list[i].netmask)==(target_ip & dl->dev_list[i].netmask))
 			return(&dl->dev_list[i]);
-	if((re=get_route(target_ip)))
+	if((re=get_route(target_ip, dl)))
 		for(i=0; i<dl->nb; i++)
 			if(!strncmp(dl->dev_list[i].name, re->ifname, IFNAMSIZ-1))
 				return(&dl->dev_list[i]);
@@ -99,18 +98,22 @@ netdev *get_suitable_dev(devlist *dl, u_int32_t target_ip)
 u_int32_t get_gateway_ip(u_int32_t dstip)
 {
 	route_elt *re;
-	if(!(re=get_route(dstip))) return(-1);
+	if(!(re=get_route(dstip, NULL))) return(-1);
 	return(re->gateway);
 }
 
-// deeply inspired by nmap
-route_elt *get_route(unsigned int dstip)
+// inspired by nmap
+route_elt *get_route(u_int32_t dstip, devlist *devicelist)
 {
-	FILE *rt;
-	char buffer[10240], *ptr, *endptr;
-	unsigned int dst, netmask, i;
+	char buffer[10240];
 	static route_elt re;
+	struct sockaddr_in sock;
+	char *ptr, *endptr;
+	unsigned int dst, netmask, i;
+	int sd, socklen=sizeof(struct sockaddr_in);
+	FILE *rt;
 
+	// search for a /proc/net/route interface
 	if((rt=fopen(FROUTE, "rt"))) {
 		fgets(buffer, sizeof(buffer), rt);
 		while(fgets(buffer, sizeof(buffer), rt)) {
@@ -131,13 +134,33 @@ route_elt *get_route(unsigned int dstip)
 			if(!ptr) continue;				// not enough fields
 			endptr=NULL;
 			netmask=strtoul(ptr, &endptr, 16);
-			if(!*ptr || *endptr) continue;			// netmask not valid
+			if(!*ptr || *endptr) continue;			// invalid netmask
 			if((dstip & netmask)==dst) {
 				fclose(rt);
 				return(&re);
 			}
 		}
 		fclose(rt);
+	} else {
+		if(!devicelist) return(NULL);
+		// use connect socket technique (taken from nmap too)
+		if((sd=socket(AF_INET, SOCK_DGRAM, 0))==-1)
+			{ perror("get_route: socket"); return(NULL); }
+		sock.sin_family=AF_INET;
+		sock.sin_addr.s_addr=dstip;
+		sock.sin_port=ntohs(53);
+		if(connect(sd, (struct sockaddr *)&sock, sizeof(struct sockaddr_in))==-1)
+			{ perror("get_route: connect"); close(sd); return(NULL); }
+		bzero(&sock, sizeof(struct sockaddr_in));
+		if(getsockname(sd, (struct sockaddr *)&sock, &socklen)==-1)
+			{ perror("get_route: getsockname"); close(sd); return(NULL); }
+		// search for a matching network interface
+		for(i=0; devicelist->dev_list[i].ip!=sock.sin_addr.s_addr && i<devicelist->nb; i++);
+		if(i==devicelist->nb) { fprintf(stderr, "get_route: not able to find a matching device\n"); close(sd); return(NULL); }
+		strncpy(re.ifname, devicelist->dev_list[i].name, IFNAMSIZ-1); re.ifname[IFNAMSIZ-1]='\0';
+		re.gateway=-1;
+		close(sd);
+		return(&re);
 	}
 	return(NULL);
 }
